@@ -1,16 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2017 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2022 -- leonerd@leonerd.org.uk
 
-package Tangence::Meta::Event;
+use v5.26;
+use Object::Pad 0.70 ':experimental(adjust_params)';
 
-use strict;
-use warnings;
-
-our $VERSION = '0.25';
-
-use Scalar::Util qw( weaken );
+package Tangence::Meta::Event 0.30;
+class Tangence::Meta::Event :strict(params);
 
 =head1 NAME
 
@@ -52,14 +49,14 @@ L<Tangence::Meta::Argument> references.
 
 =cut
 
-sub new
-{
-   my $class = shift;
-   my %args = @_;
-   $args{arguments} ||= [];
-   my $self = bless \%args, $class;
-   weaken $self->{class};
-   return $self;
+field $class :param :weak :reader;
+field $name  :param       :reader;
+field @arguments;
+
+ADJUST :params (
+   :$arguments = undef,
+) {
+   @arguments = $arguments->@* if $arguments;
 }
 
 =head1 ACCESSORS
@@ -74,12 +71,6 @@ Returns the class the event is a member of
 
 =cut
 
-sub class
-{
-   my $self = shift;
-   return $self->{class};
-}
-
 =head2 name
 
    $name = $event->name
@@ -88,12 +79,6 @@ Returns the name of the class
 
 =cut
 
-sub name
-{
-   my $self = shift;
-   return $self->{name};
-}
-
 =head2 arguments
 
    @arguments = $event->arguments
@@ -102,11 +87,7 @@ Return the arguments in a list of L<Tangence::Meta::Argument> references.
 
 =cut
 
-sub arguments
-{
-   my $self = shift;
-   return @{ $self->{arguments} };
-}
+method arguments { @arguments }
 
 =head2 argtypes
 
@@ -116,10 +97,9 @@ Return the argument types in a list of strings.
 
 =cut
 
-sub argtypes
+method argtypes
 {
-   my $self = shift;
-   return map { $_->type } $self->arguments;
+   return map { $_->type } @arguments;
 }
 
 =head1 AUTHOR
