@@ -1,17 +1,16 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2012-2017 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012-2022 -- leonerd@leonerd.org.uk
 
-package Tangence::Meta::Struct;
+use v5.26;
+use Object::Pad 0.66 ':experimental(init_expr)';
 
-use strict;
-use warnings;
+package Tangence::Meta::Struct 0.30;
+class Tangence::Meta::Struct :strict(params);
 
 use Carp;
 
-our $VERSION = '0.25';
-
 =head1 NAME
 
 C<Tangence::Meta::Struct> - structure representing one C<Tangence> structure
@@ -36,13 +35,10 @@ Returns a new instance representing the given name.
 
 =cut
 
-sub new
-{
-   my $class = shift;
-   my %args = @_;
-   my $self = bless { name => delete $args{name} }, $class;
-   return $self;
-}
+field $name    :param :reader;
+field $defined        :reader { 0 };
+
+field @fields;
 
 =head2 define
 
@@ -61,14 +57,12 @@ of L<Tangence::Meta::Field>.
 
 =cut
 
-sub define
+method define ( %args )
 {
-   my $self = shift;
-   my %args = @_;
+   $defined and croak "Cannot define $name twice";
 
-   $self->defined and croak "Cannot define ".$self->name." twice";
-
-   $self->{fields} = $args{fields};
+   $defined++;
+   @fields = @{ $args{fields} };
 }
 
 =head1 ACCESSORS
@@ -84,12 +78,6 @@ C<define>.
 
 =cut
 
-sub defined
-{
-   my $self = shift;
-   return exists $self->{fields};
-}
-
 =head2 name
 
    $name = $struct->name
@@ -98,12 +86,6 @@ Returns the name of the structure
 
 =cut
 
-sub name
-{
-   my $self = shift;
-   return $self->{name};
-}
-
 =head2 fields
 
    @fields = $struct->fields
@@ -113,11 +95,10 @@ definition.
 
 =cut
 
-sub fields
+method fields
 {
-   my $self = shift;
    $self->defined or croak $self->name . " is not yet defined";
-   return @{ $self->{fields} };
+   return @fields;
 }
 
 =head1 AUTHOR
